Model.selection <- function(object, data){
	library(hier.part)
	library(nlme)
	predictors <- dimnames(attr(object$terms, "factors"))[[2]]
	comb <-combos(length(predictors))[[1]]
	ll<-length(comb[,1])
	var.names <- vector(mode="character",length=ll)
	d <- data.frame("r.squared"=1:ll,"Adj.r.sq"=1:ll, "AIC"=1:ll, "BIC"=1:ll, "Select"=factor(rep(" ",ll),levels=c(" ","*")))
	form <- object$call$formula
	minAIC=100000
	minBIC =100000
	for (i in 1:length(comb[,1]))
	{
		pred <- predictors[comb[i,]]
		var.names[i]<-paste(pred, collapse="+") 
		#ii <- paste("`",pred,"`", collapse="+", sep="")
	      ii <- paste(pred, collapse="+", sep="")
		new_fit <-update.formula(form, paste("~  +",  ii))
		fit<-lm(new_fit, data=data)
		fit_sum <-summary(lm(fit))
		d$"r.squared"[i] <-fit_sum$"r.squared"
		d$"Adj.r.sq"[i] <-fit_sum$"adj.r.squared"
		d$"AIC"[i]<-extractAIC(fit)[2]
		if (minAIC>d$"AIC"[i]) minAIC <- d$"AIC"[i]
		d$"BIC"[i] <-BIC(fit)
		if (minBIC>d$"BIC"[i]) minBIC <- d$"BIC"[i]

	}
	rownames(d) <- var.names
	d$"Select"[d$BIC==minBIC]<-"*"
	d$"Select"[d$AIC==minAIC]<-"*"
      d
}
#use
#Model.selection(loyn.lm, loyn)



power.plot <- function(delta, sd, n)
{ 
      power <- numeric(length(n))
	j<-0
	if (length(delta)==1) constant<-"delta"
	else constant <- "sd"
	#print(constant)
	if (constant=="sd") for (i in n) {j<-j+1;power[j] <- power.t.test(n=i,sd=sd,delta=delta[1])$power}
	else for (i in n) {j<-j+1;power[j] <- power.t.test(n=i,sd=sd[1],delta=delta)$power}

	#if (kk==1){
	windows(height=4,width=4)
	old_par<- par(mar=c(4,4,2,1))
	if (constant=="sd") plot(n,power,axes=F, xlab="", ylab="", lwd=2,type="n", main=substitute(paste("sd = ", sd), list(sd=sd)))
	else plot(n,power,axes=F, xlab="", ylab="", lwd=2,type="n", main=substitute(paste("delta = ", delta), list(delta=delta)))

	axis(1, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2)
	mtext("n",1, font=2,cex=1.5, line=3)
	axis(2, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2, las=1)
	mtext("power",2, font=2,cex=1.5, line=3)
	lines(c(0,max(n)),c(0.8,0.8),lty=2)
	box(bty="l",lwd=2)

	if (constant=="sd") {
		if (length(delta)>0)
		{
			kk<-0
   			for (k in delta){
      			kk<-kk+1
   				power <- numeric(length(n))
				j<-0
				for (i in n) {j<-j+1;power[j] <- power.t.test(n=i,sd=sd,delta=delta[kk])$power}
				points(n,power,col="black", pch=16, lwd=2, type="l", lty=kk)
			}
   		}
      	legend("bottomright", title="Delta",legend=delta, lty=seq(1,length(delta),by=1))
	}
	else {
		if (length(sd)>0)
		{
			kk<-0
   			for (k in sd){
      			kk<-kk+1
   				power <- numeric(length(n))
				j<-0
				for (i in n) {j<-j+1;power[j] <- power.t.test(n=i,sd=sd[kk],delta=delta)$power}
				points(n,power,col="black", pch=16, lwd=2, type="l", lty=kk)
			}
   		}
      	legend("bottomright", title="sd",legend=sd, lty=seq(1,length(sd),by=1))
	}

}
#uses
#power.plot(delta=c(3,4,5,6),sd=2,n=seq(2,10,l=100))
#power.plot(delta=c(3.687),sd=c(1.8,2,2.2),n=seq(2,10,l=100))


mdes.plot <- function(power=.8, sd, n)
{ 
      delta <- numeric(length(n))
	j<-0
	if (length(power)==1) constant<-"power"
	else constant <- "sd"

	if (constant=="sd") for (i in n) {j<-j+1;delta[j] <- power.t.test(n=i,sd=sd,power=power[1])$delta}
	else for (i in n) {j<-j+1;delta[j] <- power.t.test(n=i,sd=sd[1],power=power)$delta}
	windows(height=4,width=4)
	old_par<- par(mar=c(4,4,2,1))
	if (constant=="sd") plot(n,delta,axes=F, xlab="", ylab="", lwd=2,type="n", main=substitute(paste("sd = ", sd), list(sd=sd)))
	else plot(n,delta,axes=F, xlab="", ylab="", lwd=2,type="n", main=substitute(paste("power = ", power), list(power=power)))

	axis(1, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2)
	mtext("n",1, font=2,cex=1.5, line=3)
	axis(2, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2, las=1)
	mtext("Minimum detectable effect size",2, font=2,cex=1.5, line=3)
	lines(c(0,max(n)),c(0.8,0.8),lty=2)
	box(bty="l",lwd=2)

	if (constant=="sd") {
		if (length(power)>0)
		{
			kk<-0
   			for (k in power){
      			kk<-kk+1
   				delta <- numeric(length(n))
				j<-0
				for (i in n) {j<-j+1;delta[j] <- power.t.test(n=i,sd=sd,power=power[kk])$delta}
				points(n,delta,col="black", pch=16, lwd=2, type="l", lty=kk)
			}
   		}
      	legend("topright", title="Power",legend=power, lty=seq(1,length(power),by=1))
	}
	else {
		if (length(sd)>0)
		{
			kk<-0
   			for (k in sd){
      			kk<-kk+1
   				delta <- numeric(length(n))
				j<-0
				for (i in n) {j<-j+1;delta[j] <- power.t.test(n=i,sd=sd[kk],power=power)$delta}
				points(n,delta,col="black", pch=16, lwd=2, type="l", lty=kk)
			}
   		}
      	legend("topright", title="sd",legend=sd, lty=seq(1,length(sd),by=1))
	}

}

#mdes.plot(power=0.8,sd=c(1.8,2,2.2),n=seq(2,10,l=100),constant="power")
#mdes.plot(power=c(.7,.8,.9),sd=c(2),n=seq(2,10,l=100),constant="sd")

power.r.plot <- function(r=NULL, power=NULL, n=NULL)
{ 
	library(pwr)
	windows(height=4,width=4)
	old_par<- par(mar=c(4,4,2,1))
     	if (length(n)>0) {
		power_val <- numeric(length(n))
		j<-0
		for (i in n) {j<-j+1;power_val[j] <- pwr.r.test(n=i,r=r)$power}
		plot(n,power_val,axes=F, xlab="", ylab="", lwd=2,type="l", main=substitute(paste("r = ", r), list(r=r)))
	}
	else {
		n_val <- numeric(length(power))
		j<-0	
		for (i in power) {j<-j+1;n_val[j] <- pwr.r.test(power=i,r=r)$n}
		plot(n_val,power,axes=F, xlab="", ylab="", lwd=2,type="l", main=substitute(paste("r = ", r), list(r=r)))
	}
	axis(1, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2)
	mtext("n",1, font=2,cex=1.5, line=3)
	axis(2, tick=T,tcl=-.2,lab=T,cex.axis=1, lwd=2, las=1)
	mtext("power",2, font=2,cex=1.5, line=3)
	lines(c(0,max(n)),c(0.8,0.8),lty=2)
	box(bty="l",lwd=2)

}
#uses
#power.r.plot(power=seq(.6,.99,l=100),r=sqrt(.92))
#power.r.plot(n=4:10,r=sqrt(.92))

Mbargraph <- function(dv,factorlist,ylabel=paste("Mean", deparse(substitute(dv))), xlabel="",type="b", ymin=NULL, ymax=NULL, errorbars="se", symbols=NULL){
  # calculate the means
  means <- tapply(dv, factorlist, mean)
  # calculate the standard deviation
  sds <- tapply(dv, factorlist, sd)
  # calculate the lengths
  ns <- tapply(dv, factorlist, length)
  # calculate the standard errors
  ses <- sds/sqrt(ns)
  # plot the bars
  ys <- pretty(c(means-ses,means+(2*ses)))
  if (type=="b"){
     if (is.list(factorlist)) {
		xs <- barplot(means,ylim=c(min(ys),max(ys)),beside=T,axes=F,xpd=F,axisnames=T,axis.lty=2,legend.text=T)
          if (xlabel=="") {
		#eval(substitute(list(Dataset$SEASON, Dataset$DENSITY))[[2]])
		xlabel=deparse(substitute(factorlist)[[3]])
		}
	}
     else {xs <- barplot(means,ylim=c(min(ys),max(ys)),beside=T,axes=F,xpd=F,axis.lty=1,legend.text=F)
		if (xlabel=="") {
			xlabel=deparse(substitute(factorlist))
 		#mtext(1,text=xlabel,line=3, cex=1.5)
		}
	}
   }
  else {
    if (is.list(factorlist)){
	xs <- barplot(means,ylim=c(min(ys),max(ys)),beside=T,axes=F,xpd=F,axis.lty=1,legend.text=F,plot=FALSE)
      interaction.plot(factorlist[[1]],factorlist[[2]], dv,ylim=c(min(ys),max(ys)))
    }
    else{
      xs <- plotMeans(means,factorlist[[1]],factorlist[[2]])
      #plot(xs,means,ylim=c(min(ys),max(ys)),axes=F,ylab="",xlab=substitute(factorlist),type="n")
      #points(xs,means,pch=16)
    }
  }
    
  # load package containing error bar function
  library(Hmisc)
  # plot the error bars<br>
  errbar(xs, means, means+ses, means-ses, add=T)
  axis(2,las=1)
  #axis(1)
  mtext(2,text=ylabel,line=3, cex=1.5)
  mtext(1,text=xlabel,line=3, cex=1.5)
  lines(c(0,max(xs)+.5),c(min(ys),min(ys)))
  Mgraph <- data.frame(xs, means=means+ses+((max(ys)-min(ys))/20))
  if (exists("symbols")){
    text(xs,means+ses, labels=symbols, pos=3)
  }
  #axis(1, at=xs,labels=c("Male", "Female"))
}

plotMeans <- function(response, factor1, factor2, error.bars = c("se", "sd", "conf.int", "none"),
    level=0.95, xlab=deparse(substitute(factor1)), ylab=paste("mean of", deparse(substitute(response))), 
    legend.lab=deparse(substitute(factor2)), main="Plot of Means",
    pch=1:n.levs.2, lty=1:n.levs.2, col=palette()){
    if (!is.numeric(response)) stop("Argument response must be numeric.")
    xlab # force evaluation
    ylab
    legend.lab
    error.bars <- match.arg(error.bars)
    if (missing(factor2)){
        if (!is.factor(factor1)) stop("Argument factor1 must be a factor.")
        valid <- !(is.na(factor1) | is.na(response))
        factor1 <- factor1[valid]
        response <- response[valid]
        means <- tapply(response, factor1, mean)
        sds <- tapply(response, factor1, sd)
        ns <- tapply(response, factor1, length)
        if (error.bars == "se") sds <- sds/sqrt(ns)
        if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
        yrange <-  if (error.bars != "none") c( min(means - sds), max(means + sds)) else range(means)
        levs <- levels(factor1)
        n.levs <- length(levs)
        plot(c(1, n.levs), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
        points(1:n.levs, means, type="b", pch=16, cex=2)
        box()
        axis(2)
        axis(1, at=1:n.levs, labels=levs)
        if (error.bars != "none") arrows(1:n.levs, means - sds, 1:n.levs, means + sds, 
            angle=90, lty=2, code=3, length=0.125)
        }
    else {
        if (!(is.factor(factor1) | is.factor(factor2))) stop("Arguments factor1 and factor2 must be factors.")
        valid <- !(is.na(factor1) | is.na(factor2) | is.na(response))
        factor1 <- factor1[valid]
        factor2 <- factor2[valid]
        response <- response[valid]
        means <- tapply(response, list(factor1, factor2), mean)
        sds <- tapply(response, list(factor1, factor2), sd)
        ns <- tapply(response, list(factor1, factor2), length)
        if (error.bars == "se") sds <- sds/sqrt(ns)
        if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
        yrange <-  if (error.bars != "none") c( min(means - sds), max(means + sds)) else range(means)
        levs.1 <- levels(factor1)
        levs.2 <- levels(factor2)
        n.levs.1 <- length(levs.1)
        n.levs.2 <- length(levs.2)
        if (n.levs.2 > length(col)) stop(paste("Number of groups for factor2, ", n.levs.2,
            ", exceeds number of distinct colours, ", length(col), ".", sep=""))
        plot(c(1, n.levs.1 + 1), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
        box()
        axis(2)
        axis(1, at=1:n.levs.1, labels=levs.1)
        for (i in 1:n.levs.2){
            points(1:n.levs.1, means[, i], type="b", pch=pch[i], cex=2, col=col[i], lty=lty[i])
            if (error.bars != "none") arrows(1:n.levs.1, means[, i] - sds[, i], 
                1:n.levs.1, means[, i] + sds[, i], angle=90, code=3, col=col[i], lty=lty[i], length=0.125)
            }
        x.posn <- n.levs.1 + 0.25
        y.posn <- sum(c(0.1, 0.9) * par("usr")[c(3,4)])
        text(x.posn, y.posn, legend.lab, adj=c(0, -.5))
        legend(x.posn, y.posn, levs.2, pch=pch, col=col, lty=lty)
        }
    invisible(NULL)
    }

