#you need to provide it with one of the following classes of denom
# 1. a summary.aov (list of anova objects) - Residuals are extracted from each ANOVA strata
# 2. a list (a list of anova objects) - Residuals are extracted from each ANOVA strata
# 3. anova (a single anova object) - Residuals are extracted from single anova
# 4. character (a vector of characters) - list of Residual terms (from within current model)
Anovam <- function(object, denoms="Resid") {
		#print(class(denoms))
                 #                           print(denoms)
		if (length(object[[1]])>1) a<-object[[1]]
                else {
                  a<-object[[1]][[1]]
                  if (is.null(a)) a<-object[[1]]
                }
                
		var_names<- rownames(a)
                
		terms<-grep("^[^ ]",rownames(a))
		var_length <- length(var_names)-1
                 
		if(match("summary.aov",class(denoms), nomatch=0)) #if denoms presents as a list of anova objects
                  {
                    
                    a[length(rownames(a)),"Sum Sq"]<-denoms[[1]]["Residuals", "Sum Sq"]
                    #a["Residuals","Df"]<-denoms[[1]]["Residuals", "Df"]
                    #a["Residuals","Mean Sq"]<-denoms[[1]]["Residuals", "Sum Sq"]/denoms[[1]]["Residuals","Df"]
                    a[length(rownames(a)),"Df"]<-denoms[[1]]["Residuals", "Df"]
                    a[length(rownames(a)),"Mean Sq"]<-denoms[[1]]["Residuals", "Sum Sq"]/denoms[[1]]["Residuals","Df"]
              
                    b<-denoms[[1]]["Resid",]
		}
		if (match("list",class(denoms), nomatch=0)) #if denoms presents as a list of anova objects
		{
			b<-NULL
			for(i in 1:length(denoms)) 	b<-rbind(b,denoms[[i]])
			a[length(rownames(a)),"Sum Sq"]<-b[length(denoms),"Sum Sq"]	
			a[length(rownames(a)),"Df"]<-b[length(denoms),"Df"]
			a[length(rownames(a)),"Mean Sq"]<-b[length(denoms),"Sum Sq"]/b[length(denoms),"Df"]
		}
                   #print("list")
                    #print(a)
		if (match("anova",class(denoms),nomatch=0)) #if denoms presents as a single ANOVA table
		{
			if(dim(denoms)[1]!=1) denoms<-denoms[rownames(denoms)[length(rownames(denoms))],]
			b<-denoms
			bb<-b
			if(dim(b)[1]==1) for(i in 1:var_length) bb<-rbind(bb,b)
			b<-bb
			a[length(rownames(a)),"Sum Sq"]<-b[var_length,"Sum Sq"]	
			a[length(rownames(a)),"Df"]<-b[var_length,"Df"]
			a[length(rownames(a)),"Mean Sq"]<-b[var_length,"Sum Sq"]/b[var_length,"Df"]
		}	
                     # print("anova")
                    #print(a)
		if (match("character",class(denoms),nomatch=0)) #if a vector of characters
		{
			b<-a
			if(length(denoms)==1 | length(denoms)==length(terms)-1 | length(denoms)==var_length) {}
			else stop("The number of denominators is not a function of the number of tests")
			if(length(denoms)==1) denoms<-rep(denoms, var_length)
			if(length(denoms)>length(terms)) terms<-grep("",rownames(a))
			b<-a[denoms,]
		}
		
                     #print("char")
                    #print(a)                 
		for (i in 1:var_length) {
			if (!is.na(match(i,terms))) j<-match(i,terms)
			SS<-a[i,"Sum Sq"]
			MS<-SS/a[i,"Df"]
			#a[i,"F value"] <- f <-MS/b[denoms[j],"Mean Sq"]
			#a[i,"Pr(>F)"] <- 1-pf(f,a[i,"Df"],b[denoms[j],"Df"])
			a[i,"F value"] <- f <-MS/b[j,"Mean Sq"]
			a[i,"Pr(>F)"] <- 1-pf(f,a[i,"Df"],b[j,"Df"])
		}
		a
	}


AnovaM.aov<- function(object, type="I",error,intercept = FALSE, split=NULL, expand.split = TRUE, 
    keep.zero.df = TRUE, denoms="Resid", RM=F,...)
  {
    require(car)
    dots <- list(...)
    op<-options()
    options(warn=-1)
                                #Create the base summary ANOVA table                                            
    if(!is.null(split))
      {
        s<-do.call("summary", c(list(object, intercept=intercept, split=split, expand.split=expand.split, keep.zero.df=keep.zero.df, dots)))
      }
    else{
      s<-do.call("summary", c(list(object,intercept=intercept,expand.split=expand.split,keep.zero.df=keep.zero.df)))
                                        #s<-do.call("summary", c(list(object, intercept, expand.split=F, keep.zero.df, dots)))
    }
    #Get a list of the factor names                                            
    names<-character(0)
    if (length(s)>1) #if there are multiple strata
      {
	for (i in 1:length(s))
          {
                                        #if it is a aovlist
            nms<-attr(s[[i]][[1]],"row.names")
            nms <- sub("^ +", "", nms)
            nms <- sub(" +$", "", nms)
                                        #ignore the one called residuals
            nms<-nms[-length(nms)]
            names<-c(names,nms)
          }
      }
    else
      {
        if (length(s[[1]])==1) nms<-attr(s[[1]][[1]],"row.names")
        else nms<-attr(s[[1]],"row.names")
       
        nms <- sub("^ +", "", nms)
        nms <- sub(" +$", "", nms)
             #ignore the one called residuals
        nms<-nms[-length(nms)]
        names<-c(names,nms)
        if (!missing(error))  #if the user has defined a aov model to use as the residual term
          {
            sumry<-summary(error, corr=FALSE)
            s2<-sumry$sigma^2
            error.df<-error$df.residual
            nms<-row.names(s[[1]])
            nms <- sub("^ +", "", nms)
            nms <- sub(" +$", "", nms)
            ind<-match("Residuals",nms, nomatch=0)
            s[[1]][ind,"Df"]<-error.df
            s[[1]][ind,"Sum Sq"]<-s2*error.df
            s[[1]][ind,"Mean Sq"]<-s2	
          }
      }
                                  #Tease out the components of the call
    if (length(s)>1) form<-attr(object,"call")
    else {
      form<-object$call
      if (is.null(form)) form<-attr(object,"call")
    }
    original_form<- form
    form<-deparse(form)
    form<-sub("Error\\(([A-Za-z0-9_\\.\\/\\,\\ \\(\\)]*)\\)","\\1",form)
 
    #put in the calculations of Greenhouse Geiser and Huhn-Feldt epsilons
    if (RM==T) {
      gg.hf <- splitPlot.to.RepeatedMeasures(object)
      print(gg.hf)
#      gg.hf<-NULL
#      call<-attr(object, "call")
#      block<-names(object)[2]
#      resp<-as.character(call[[2]][[2]])
#      rept<-row.names(summary(object[[3]])[[1]])
#      rept_lab<- trim.blanks(rept[-grep("Residuals",rept)])
#      rept<-trim.blanks(rept[-grep(":|Residuals",rept)])
#      data<-eval(call[[3]])
#      between<-row.names(summary(object[[2]])[[1]])
#      between_lab<- trim.blanks(between[-grep("Residuals",between)])
#      between<-trim.blanks(between[-grep(":|Residuals",between)])
#                                        #data.rm<-reshape(call[[3]],direction="wide", timevar=rept,idvar=block, v.names=resp)
#      data.rm<-reshape(data,direction="wide", timevar=rept,idvar=block, v.names=resp)
#      if (length(between_lab)==0) {
#        data.manova<-lm(as.matrix(data.rm[grep(resp, colnames(data.rm))])~1)
#      }
#      else {
#        between<-eval(parse(text=paste("data.rm$",between,sep="")))
#        data.manova<-lm(as.matrix(data.rm[grep(resp, colnames(data.rm))])~between)
#      }
#      
#      idata <- data.frame(within=as.factor(levels(eval(parse(text=paste(as.character(call[[3]]),"$",rept, sep=""))))))
#      data.ok<-Anova(data.manova, idata=idata, idesign=~within, type="III")
#      gg.hf<-GGHF(data.ok)
#      colnames(gg.hf)<-c("Greenhouse-Geisser", "Hunyh-Feldt")
#      rownames(gg.hf) <- c(between_lab, rept_lab)
#      cat("Epsilon values\n")
#      print(gg.hf)
#      
    }
      
	#consider this one instead!!
	#form<-sub("Error","",form)
	form<-eval(parse(text=form))
    #print(form)
	x<-model.matrix(form)
	offset <- model.offset(model.frame(form))
	asgn <- attr(x, "assign")
	tl <- attr(form$terms, "term.labels")
	fac<-attr(form$terms, "factors")
	split.fact<- vector("list", length(tl))
	names(split.fact)<-tl
    
	for (k in 1:length(tl)) {
		scope <- attr(terms(update.formula(form, paste("~",tl[k],sep=""))), "term.labels")
  		ndrop <- match(scope, tl)
   		ii<-seq(along = asgn)[asgn == ndrop]
		split.fact[k]<- vector("list",length(ii))
        	split.fact[k]<-list(ii)	
	}

	names.scope<- vector("list", length(names))
	names(names.scope)<-names
	main.scope<-names.scope
	for (ll in 1:length(names)){
		for (mm in 1:length(split.fact)){
			if (names(names.scope[ll])==names(split.fact[mm])) {
				names.scope[[ll]]<-split.fact[[mm]] #vector("list",split.fact[[mm]])
				main.scope[[ll]]<-names(split.fact[mm])
			}
		}
	}

	
if(!is.null(split)){
	split.cont <- split
	split.cont <- splitInteractions(split.cont, factors=attr(form$terms,"factors"), c("(Intercept)", tl), asgn, names(form$coef))

	for (ll in 1:length(names)){
		for (mm in 1:length(split.cont)){
			for (nn in 1:length(split.cont[[mm]])){
				if (names(names.scope[ll])==paste(names(split.cont[mm]),": ",names(split.cont[[mm]][nn]),sep="")){
					names.scope[[ll]]<-split.fact[[names(split.cont[mm])]][split.cont[[mm]][[nn]]]  #,split.cont[[mm]][[nn]]]
					main.scope[[ll]]<-names(split.cont[mm])
				}
			}		
		}
	
	}
}

	chisq <- deviance(form)
  #from full model (for type III SS)
	y <- form$residuals + predict(form)
	tol<-.Machine$double.eps^0.5

	for (ll in 1:length(names.scope)){
		if(type=="II"){
			if(ncol(fac)>1) {
				ndrop<-relatives(main.scope[[ll]],main.scope,fac)
				ii<-seq(along = asgn)[asgn == ndrop]
				jj <- setdiff(seq(ncol(x)), ii)
			}
			else jj<-seq(ncol(x))
			
			z<-lm.fit(x[, jj, drop = FALSE], y, offset = offset)
			oldClass(z) <- "lm"
			chisq <- deviance(z)
			jj <- setdiff(jj, names.scope[[ll]])
			z<-lm.fit(x[, jj, drop = FALSE], y, offset = offset)
			oldClass(z) <- "lm"
			RSS <- deviance(z)
		}
		if(type=="III"){
	
			jj <- setdiff(seq(ncol(x)), names.scope[[ll]])
			z<-lm.fit(x[, jj, drop = FALSE], y, offset = offset)
			oldClass(z) <- "lm"
			RSS <- deviance(z)
		}
                                          
		if (type!="I") SS<-RSS-chisq
                                        
		if(length(s)>1 & type!="I"){
 		for (mm in 1:length(s)){
			nms<-row.names(s[[mm]][[1]])
			nms <- sub("^ +", "", nms)
			nms <- sub(" +$", "", nms)
			ind<-match(names(names.scope[ll]),nms, nomatch=0)
			if (ind && SS>tol) {
				s[[mm]][[1]][ind,"Sum Sq"]<-SS
				s[[mm]][[1]][ind,"Mean Sq"]<-SS/s[[mm]][[1]][ind,"Df"]
				s[[mm]][[1]][ind,"F value"]<-s[[mm]][[1]][ind,"Mean Sq"]/s[[mm]][[1]]["Residuals","Mean Sq"]
				s[[mm]][[1]][ind,"Pr(>F)"]<-pf(s[[mm]][[1]][ind,"F value"], s[[mm]][[1]][ind,"Df"], s[[mm]][[1]]["Residuals","Df"], lower.tail = FALSE)
                                #     print(names(names.scope[ll]))       
                                #print(gg.hf[ind,1])
                                #print(rownames(s[[mm]][[1]][ind,]))
                                #print(rownames(gg.hf))
                                #print(match(trim.blanks(rownames(s[[mm]][[1]][ind,])),rownames(gg.hf)))
                               if (RM==T && 1==2) {
                                if (!is.na(match(trim.blanks(rownames(s[[mm]][[1]][ind,])),rownames(gg.hf))))  
                                  mtch<-match(trim.blanks(rownames(s[[mm]][[1]][ind,])),rownames(gg.hf))
                                if (is.na(mtch)) mtch<-1            
                                if (RM==T && gg.hf[mm][1]!="NA")
                                  {   
                                    #mtch <- match(rownames(s[[mm]][[1]][ind,]),rownames(gg.hf))
                                # print(mtch)
                                    eps.GG<-gg.hf[mtch][1]
                                    dfsGR_df<-s[[mm]][[1]]["Residuals", "Df"]
                                    dfsG<-s[[mm]][[1]][ind, "Df"]*eps.GG
                                    dfsH<-s[[mm]][[1]][ind, "Df"]*gg.hf[mtch,2]
                                    s[[mm]][[1]][ind,"GG.P"]<-1-pf(s[[mm]][[1]][ind,"F value"],dfsG,dfsGR_df)
                                    s[[mm]][[1]][ind,"HF.P"]<-1-pf(s[[mm]][[1]][ind,"F value"],dfsH,dfsGR_df)
                                    
                                }
                              }
			}
			else next	
		}
		}
		else{
                  if (type!="I") {
			nms<-row.names(s[[1]])
			nms <- sub("^ +", "", nms)
			nms <- sub(" +$", "", nms)
			ind<-match(names(names.scope[ll]),nms, nomatch=0)
			if (ind && SS>tol) {
				s[[1]][ind,"Sum Sq"]<-SS
				s[[1]][ind,"Mean Sq"]<-SS/s[[1]][ind,"Df"]
				s[[1]][ind,"F value"]<-s[[1]][ind,"Mean Sq"]/s[[1]]["Residuals","Mean Sq"]
				s[[1]][ind,"Pr(>F)"]<-pf(s[[1]][ind,"F value"], s[[1]][ind,"Df"], s[[1]]["Residuals","Df"], lower.tail = FALSE)
			}
                      }
		}
	}

    if (length(s)>1){
          ss<-NULL
          for (i in 1:length(s)){
            #print(denoms[[i]])
            #print(s[[i]])

                                        #print(paste("denoms=",denoms[[i-1]]))
            #ss<-list(ss,Anovam(s[[i]], denoms=denoms[[i]]))
            if (denoms!="Resid") s[[i]]<-Anovam(s[[i]], denoms=denoms[[i]])
            #ss<-list(ss,s[[i]], denoms=denoms)
          }
          #s<-ss
        }
        else if (denoms!="Resid") s<-Anovam(s, denoms=denoms)
  
    #now correct for sphericity if RM=T
    
    if (RM==T) {
      if(length(s)>1){
        for (ll in 1:length(names.scope)){
          for (mm in 1:length(s)){
            nms<-row.names(s[[mm]][[1]])
            nms <- sub("^ +", "", nms)
            nms <- sub(" +$", "", nms)
            ind<-match(names(names.scope[ll]),nms, nomatch=0)
            if (!ind) next
            if (!is.na(match(trim.blanks(rownames(s[[mm]][[1]][ind,])),rownames(gg.hf))))  
              mtch<-match(trim.blanks(rownames(s[[mm]][[1]][ind,])),rownames(gg.hf))
            if (is.na(mtch)) mtch<-1
            
            if (RM==T && !is.na(gg.hf[mm][1]))
              {   
                eps.GG<-gg.hf[mtch][1]
                dfsGR_df<-s[[mm]][[1]]["Residuals", "Df"]
                dfsG<-s[[mm]][[1]][ind, "Df"]*eps.GG
                dfsH<-s[[mm]][[1]][ind, "Df"]*gg.hf[mtch,2]
                s[[mm]][[1]][ind,"GG.P"]<-1-pf(s[[mm]][[1]][ind,"F value"],dfsG,dfsGR_df)
                s[[mm]][[1]][ind,"HF.P"]<-1-pf(s[[mm]][[1]][ind,"F value"],dfsH,dfsGR_df)         
              }
          }
        }
      }
    }
    

    
                                        #var_names<- rownames(s[[1]])
	#terms<-grep("^[^ ]",rownames(s[[1]]))
	#var_length <- length(var_names)-1
	#if(length(denoms)==1) denoms<-rep(denoms, var_length)
	#for (i in 1:var_length) {
#		if (!is.na(match(i,terms))) j<-match(i,terms)
#		SS<-s[[1]][i,"Sum Sq"]
#		MS<-SS/s[[1]][i,"Df"]
#		s[[1]][i,"F value"] <- f <-MS/s[[1]][denoms[j],"Mean Sq"]
#		s[[1]][i,"Pr(>F)"] <- 1-pf(f,s[[1]][i,"Df"],s[[1]][denoms[j],"Df"])
#	}

	options(op)
	#if(length(s)<2) class(s)<-c("anova","data.frame")
	attr(s, "heading") <- c("Anova Table (Type III tests)\n", paste("Response:", responseName(object)))
        attr(s, "call") <- original_form
        s
        
        #print(gg.hf)
        s

}


summaryRM1<- function(object) {
  call<-attr(object, "call")
  data<- call[[3]]
  
  #sum <- object #summary(object,...)
  #eps <- epsi.GG.HF(rm)
  #eps.GG <- eps[1]
  #eps.HF <- eps[2]
  #term.length <- length(sum[[2]][[1]]$Df)
  #dfsG <- sum[[2]][[1]]$Df*eps.GG
  #dfsH <- sum[[2]][[1]]$Df*eps.HF
  #GG.p <- 1-pf(sum[[2]][[1]]$"F value",dfsG,dfsG[term.length])
  #HF.p <- 1-pf(sum[[2]][[1]]$"F value",dfsH,dfsH[term.length])
  #sum[[2]][[1]]$"GG.P" <- GG.p
  #sum[[2]][[1]]$HF.P <- HF.p
  #print(sum,digits=5)
  #cat(paste("\n","GG epsilon: ",format(eps.GG,digits=7),"\n",sep=""))
  #cat(paste("HF epsilon: ",format(eps.HF,digits=7),"\n"))
}



GGHF <- function(object) #requires a fitted model (aovlist)
{
  nterms <- length(object$terms)
  GG <- function(SSPE, P) {
    p <- nrow(SSPE)
    if (p < 2) 
      return(NA)
    lambda <- eigen(SSPE %*% solve(t(P) %*% P))$values
    lambda <- lambda[lambda > 0]
    ((sum(lambda)/p)^2)/(sum(lambda^2)/p)
  }
  HF <- function(gg, error.df, p) {
    ((error.df + 1) * p * gg - 2)/(p * (error.df - p * gg))
  }
  gg.hf<-NULL
  for (term in 2:nterms) {
    SSP <- object$SSP[[term]]
    SSPE <- object$SSPE[[term]]
    P <- object$P[[term]]
    p <- ncol(P)
    gg <- GG(SSPE, P)
    error.df <- object$error.df
    hf<-HF(gg, error.df, p)
    gg.hf<-rbind(gg.hf,cbind(gg,hf))
  }
  gg.hf
}

AnovaM.aovlist<- function(object, ...) {
	AnovaM.aov(object,...)
}


AnovaM<-function (mod, type = "I", denoms="Resid", split=list(),...) 
{
	dots <- list(...)
 
    if (type == "IIII") {
        if (suppressWarnings(!mod == "aovlist" && is.null(split))) {
            #mod <- aov(mod)
            #UseMethod("anova", mod)
			mod<- do.call("anova", c(list(mod, dots)))
			Anovam(mod, denoms=denoms)
        }
        else {
            #mod <- aov(mod)
            #UseMethod("summary", mod)
			mod<- do.call("summary", c(list(mod,intercept = FALSE, split=split, 
    					expand.split = TRUE, keep.zero.df = TRUE,dots)))			
			Anovam(mod, denoms=denoms)
        }
	}
    else UseMethod("AnovaM", mod)
}


splitPlot.to.RepeatedMeasures <- function(object){
  gg.hf <- NULL
  call<-attr(object, "call")
  block<-names(object)[2]
  resp<-as.character(call[[2]][[2]])
  rept<-row.names(summary(object[[3]])[[1]])
  rept_lab<- trim.blanks(rept[-grep("Residuals",rept)])
  rept<-trim.blanks(rept[-grep(":|Residuals",rept)])
  #form<-sub("Error\\(([A-Za-z0-9_\\.\\/\\,\\ \\(\\)]*)\\)","\\1",form)
  #rept<-sub("C(\\([A-Za-z_\\.]*\\), poly, 1)","\\1",rept)
  #rept<-sub("C(\\(YEAR\\).*)","\\1",rept)
  rept<-sub("C\\(([A-Za-z_\\.]*).*\\)","\\1",rept)
  #print(rept)
  data<-eval(call[[3]])
  between<-row.names(summary(object[[2]])[[1]])
  between_lab<- trim.blanks(between[-grep("Residuals",between)])
  between<-trim.blanks(between[-grep(":|Residuals",between)])
                                        #data.rm<-reshape(call[[3]],direction="wide", timevar=rept,idvar=block, v.names=resp)
  data.rm<-reshape(data,direction="wide", timevar=rept,idvar=block, v.names=resp)
  if (length(between_lab)==0) {
    data.manova<-lm(as.matrix(data.rm[grep(resp, colnames(data.rm))])~1)
  }
  else {
    between<-eval(parse(text=paste("data.rm$",between,sep="")))
    data.manova<-lm(as.matrix(data.rm[grep(resp, colnames(data.rm))])~between)
  }
  
  idata <- data.frame(within=as.factor(levels(eval(parse(text=paste(as.character(call[[3]]),"$",rept, sep=""))))))
  data.ok<-Anova(data.manova, idata=idata, idesign=~within, type="III")
  gg.hf<-GGHF(data.ok)
  colnames(gg.hf)<-c("Greenhouse-Geisser", "Hunyh-Feldt")
  rownames(gg.hf) <- c(between_lab, rept_lab)
  cat("Epsilon values\n")
  #print(gg.hf)
  gg.hf
}
